from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.household import Household


T = TypeVar("T", bound="InventoryLocation")


@_attrs_define
class InventoryLocation:
    """Moves `UniqueValidator`'s from the validation stage to the save stage.
    It solves the problem with nested validation for unique fields on update.

    If you want more details, you can read related issues and articles:
    https://github.com/beda-software/drf-writable-nested/issues/1
    http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

    Example of usage:
    ```
        class Child(models.Model):
        field = models.CharField(unique=True)


    class Parent(models.Model):
        child = models.ForeignKey('Child')


    class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
        class Meta:
            model = Child


    class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
        child = ChildSerializer()

        class Meta:
            model = Parent
    ```

    Note: `UniqueFieldsMixin` must be applied only on the serializer
    which has unique fields.

    Note: When you are using both mixins
    (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
    you should put `UniqueFieldsMixin` ahead.

        Attributes:
            name (str):
            household (Household): Adds nested create feature
            id (int | Unset):
            is_freezer (bool | Unset):
    """

    name: str
    household: Household
    id: int | Unset = UNSET
    is_freezer: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        household = self.household.to_dict()

        id = self.id

        is_freezer = self.is_freezer

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "household": household,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if is_freezer is not UNSET:
            field_dict["is_freezer"] = is_freezer

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("name", (None, str(self.name).encode(), "text/plain")))

        files.append(("household", (None, json.dumps(self.household.to_dict()).encode(), "application/json")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.is_freezer, Unset):
            files.append(("is_freezer", (None, str(self.is_freezer).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.household import Household

        d = dict(src_dict)
        name = d.pop("name")

        household = Household.from_dict(d.pop("household"))

        id = d.pop("id", UNSET)

        is_freezer = d.pop("is_freezer", UNSET)

        inventory_location = cls(
            name=name,
            household=household,
            id=id,
            is_freezer=is_freezer,
        )

        inventory_location.additional_properties = d
        return inventory_location

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.booking_type_enum import BookingTypeEnum
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.inventory_entry import InventoryEntry
    from ..models.inventory_location import InventoryLocation


T = TypeVar("T", bound="InventoryLog")


@_attrs_define
class InventoryLog:
    """
    Attributes:
        entry (InventoryEntry): Adds nested create feature
        old_inventory_location (InventoryLocation): Moves `UniqueValidator`'s from the validation stage to the save
            stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        new_inventory_location (InventoryLocation): Moves `UniqueValidator`'s from the validation stage to the save
            stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        created_at (datetime.datetime):
        id (int | Unset):
        booking_type (BookingTypeEnum | Unset): * `add` - Add
            * `remove` - Remove
            * `move` - Move
        old_amount (float | Unset):
        new_amount (float | Unset):
        note (None | str | Unset):
    """

    entry: InventoryEntry
    old_inventory_location: InventoryLocation
    new_inventory_location: InventoryLocation
    created_at: datetime.datetime
    id: int | Unset = UNSET
    booking_type: BookingTypeEnum | Unset = UNSET
    old_amount: float | Unset = UNSET
    new_amount: float | Unset = UNSET
    note: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        entry = self.entry.to_dict()

        old_inventory_location = self.old_inventory_location.to_dict()

        new_inventory_location = self.new_inventory_location.to_dict()

        created_at = self.created_at.isoformat()

        id = self.id

        booking_type: str | Unset = UNSET
        if not isinstance(self.booking_type, Unset):
            booking_type = self.booking_type.value

        old_amount = self.old_amount

        new_amount = self.new_amount

        note: None | str | Unset
        if isinstance(self.note, Unset):
            note = UNSET
        else:
            note = self.note

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "entry": entry,
                "old_inventory_location": old_inventory_location,
                "new_inventory_location": new_inventory_location,
                "created_at": created_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if booking_type is not UNSET:
            field_dict["booking_type"] = booking_type
        if old_amount is not UNSET:
            field_dict["old_amount"] = old_amount
        if new_amount is not UNSET:
            field_dict["new_amount"] = new_amount
        if note is not UNSET:
            field_dict["note"] = note

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.inventory_entry import InventoryEntry
        from ..models.inventory_location import InventoryLocation

        d = dict(src_dict)
        entry = InventoryEntry.from_dict(d.pop("entry"))

        old_inventory_location = InventoryLocation.from_dict(d.pop("old_inventory_location"))

        new_inventory_location = InventoryLocation.from_dict(d.pop("new_inventory_location"))

        created_at = isoparse(d.pop("created_at"))

        id = d.pop("id", UNSET)

        _booking_type = d.pop("booking_type", UNSET)
        booking_type: BookingTypeEnum | Unset
        if isinstance(_booking_type, Unset):
            booking_type = UNSET
        else:
            booking_type = BookingTypeEnum(_booking_type)

        old_amount = d.pop("old_amount", UNSET)

        new_amount = d.pop("new_amount", UNSET)

        def _parse_note(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        note = _parse_note(d.pop("note", UNSET))

        inventory_log = cls(
            entry=entry,
            old_inventory_location=old_inventory_location,
            new_inventory_location=new_inventory_location,
            created_at=created_at,
            id=id,
            booking_type=booking_type,
            old_amount=old_amount,
            new_amount=new_amount,
            note=note,
        )

        inventory_log.additional_properties = d
        return inventory_log

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="UserSpaceBatchUpdate")


@_attrs_define
class UserSpaceBatchUpdate:
    """
    Attributes:
        user_spaces (list[int]):
        group_set (list[int]):
        household (int | None | Unset):
    """

    user_spaces: list[int]
    group_set: list[int]
    household: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        user_spaces = self.user_spaces

        group_set = self.group_set

        household: int | None | Unset
        if isinstance(self.household, Unset):
            household = UNSET
        else:
            household = self.household

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user_spaces": user_spaces,
                "group_set": group_set,
            }
        )
        if household is not UNSET:
            field_dict["household"] = household

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        for user_spaces_item_element in self.user_spaces:
            files.append(("user_spaces", (None, str(user_spaces_item_element).encode(), "text/plain")))

        for group_set_item_element in self.group_set:
            files.append(("group_set", (None, str(group_set_item_element).encode(), "text/plain")))

        if not isinstance(self.household, Unset):
            if isinstance(self.household, int):
                files.append(("household", (None, str(self.household).encode(), "text/plain")))
            else:
                files.append(("household", (None, str(self.household).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        user_spaces = cast(list[int], d.pop("user_spaces"))

        group_set = cast(list[int], d.pop("group_set"))

        def _parse_household(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        household = _parse_household(d.pop("household", UNSET))

        user_space_batch_update = cls(
            user_spaces=user_spaces,
            group_set=group_set,
            household=household,
        )

        user_space_batch_update.additional_properties = d
        return user_space_batch_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    from_date: str | Unset = UNSET,
    meal_type: list[str] | Unset = UNSET,
    to_date: str | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    params["from_date"] = from_date

    json_meal_type: list[str] | Unset = UNSET
    if not isinstance(meal_type, Unset):
        json_meal_type = meal_type

    params["meal_type"] = json_meal_type

    params["to_date"] = to_date

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/meal-plan/ical/",
        "params": params,
    }

    return _kwargs


def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> str | None:
    if response.status_code == 200:
        response_200 = response.text
        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[str]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    from_date: str | Unset = UNSET,
    meal_type: list[str] | Unset = UNSET,
    to_date: str | Unset = UNSET,
) -> Response[str]:
    """logs request counts to redis cache total/per user/

    Args:
        from_date (str | Unset):
        meal_type (list[str] | Unset):
        to_date (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[str]
    """

    kwargs = _get_kwargs(
        from_date=from_date,
        meal_type=meal_type,
        to_date=to_date,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    from_date: str | Unset = UNSET,
    meal_type: list[str] | Unset = UNSET,
    to_date: str | Unset = UNSET,
) -> str | None:
    """logs request counts to redis cache total/per user/

    Args:
        from_date (str | Unset):
        meal_type (list[str] | Unset):
        to_date (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        str
    """

    return sync_detailed(
        client=client,
        from_date=from_date,
        meal_type=meal_type,
        to_date=to_date,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    from_date: str | Unset = UNSET,
    meal_type: list[str] | Unset = UNSET,
    to_date: str | Unset = UNSET,
) -> Response[str]:
    """logs request counts to redis cache total/per user/

    Args:
        from_date (str | Unset):
        meal_type (list[str] | Unset):
        to_date (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[str]
    """

    kwargs = _get_kwargs(
        from_date=from_date,
        meal_type=meal_type,
        to_date=to_date,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    from_date: str | Unset = UNSET,
    meal_type: list[str] | Unset = UNSET,
    to_date: str | Unset = UNSET,
) -> str | None:
    """logs request counts to redis cache total/per user/

    Args:
        from_date (str | Unset):
        meal_type (list[str] | Unset):
        to_date (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        str
    """

    return (
        await asyncio_detailed(
            client=client,
            from_date=from_date,
            meal_type=meal_type,
            to_date=to_date,
        )
    ).parsed



from __future__ import annotations

import datetime
import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.food import Food
    from ..models.inventory_location import InventoryLocation
    from ..models.unit import Unit


T = TypeVar("T", bound="PatchedInventoryEntry")


@_attrs_define
class PatchedInventoryEntry:
    """Adds nested create feature

    Attributes:
        id (int | Unset):
        inventory_location (InventoryLocation | Unset): Moves `UniqueValidator`'s from the validation stage to the save
            stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        sub_location (None | str | Unset):
        code (None | str | Unset):
        food (Food | Unset): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        unit (Unit | Unset): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        amount (float | Unset):
        expires (datetime.date | None | Unset):
        note (None | str | Unset):
        label (str | Unset):
        created_at (datetime.datetime | Unset):
        created_by (int | Unset):
    """

    id: int | Unset = UNSET
    inventory_location: InventoryLocation | Unset = UNSET
    sub_location: None | str | Unset = UNSET
    code: None | str | Unset = UNSET
    food: Food | Unset = UNSET
    unit: Unit | Unset = UNSET
    amount: float | Unset = UNSET
    expires: datetime.date | None | Unset = UNSET
    note: None | str | Unset = UNSET
    label: str | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    created_by: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        inventory_location: dict[str, Any] | Unset = UNSET
        if not isinstance(self.inventory_location, Unset):
            inventory_location = self.inventory_location.to_dict()

        sub_location: None | str | Unset
        if isinstance(self.sub_location, Unset):
            sub_location = UNSET
        else:
            sub_location = self.sub_location

        code: None | str | Unset
        if isinstance(self.code, Unset):
            code = UNSET
        else:
            code = self.code

        food: dict[str, Any] | Unset = UNSET
        if not isinstance(self.food, Unset):
            food = self.food.to_dict()

        unit: dict[str, Any] | Unset = UNSET
        if not isinstance(self.unit, Unset):
            unit = self.unit.to_dict()

        amount = self.amount

        expires: None | str | Unset
        if isinstance(self.expires, Unset):
            expires = UNSET
        elif isinstance(self.expires, datetime.date):
            expires = self.expires.isoformat()
        else:
            expires = self.expires

        note: None | str | Unset
        if isinstance(self.note, Unset):
            note = UNSET
        else:
            note = self.note

        label = self.label

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        created_by = self.created_by

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if inventory_location is not UNSET:
            field_dict["inventory_location"] = inventory_location
        if sub_location is not UNSET:
            field_dict["sub_location"] = sub_location
        if code is not UNSET:
            field_dict["code"] = code
        if food is not UNSET:
            field_dict["food"] = food
        if unit is not UNSET:
            field_dict["unit"] = unit
        if amount is not UNSET:
            field_dict["amount"] = amount
        if expires is not UNSET:
            field_dict["expires"] = expires
        if note is not UNSET:
            field_dict["note"] = note
        if label is not UNSET:
            field_dict["label"] = label
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if created_by is not UNSET:
            field_dict["created_by"] = created_by

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.inventory_location, Unset):
            files.append(
                (
                    "inventory_location",
                    (None, json.dumps(self.inventory_location.to_dict()).encode(), "application/json"),
                )
            )

        if not isinstance(self.sub_location, Unset):
            if isinstance(self.sub_location, str):
                files.append(("sub_location", (None, str(self.sub_location).encode(), "text/plain")))
            else:
                files.append(("sub_location", (None, str(self.sub_location).encode(), "text/plain")))

        if not isinstance(self.code, Unset):
            if isinstance(self.code, str):
                files.append(("code", (None, str(self.code).encode(), "text/plain")))
            else:
                files.append(("code", (None, str(self.code).encode(), "text/plain")))

        if not isinstance(self.food, Unset):
            files.append(("food", (None, json.dumps(self.food.to_dict()).encode(), "application/json")))

        if not isinstance(self.unit, Unset):
            files.append(("unit", (None, json.dumps(self.unit.to_dict()).encode(), "application/json")))

        if not isinstance(self.amount, Unset):
            files.append(("amount", (None, str(self.amount).encode(), "text/plain")))

        if not isinstance(self.expires, Unset):
            if isinstance(self.expires, datetime.date):
                files.append(("expires", (None, self.expires.isoformat().encode(), "text/plain")))
            else:
                files.append(("expires", (None, str(self.expires).encode(), "text/plain")))

        if not isinstance(self.note, Unset):
            if isinstance(self.note, str):
                files.append(("note", (None, str(self.note).encode(), "text/plain")))
            else:
                files.append(("note", (None, str(self.note).encode(), "text/plain")))

        if not isinstance(self.label, Unset):
            files.append(("label", (None, str(self.label).encode(), "text/plain")))

        if not isinstance(self.created_at, Unset):
            files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.created_by, Unset):
            files.append(("created_by", (None, str(self.created_by).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.food import Food
        from ..models.inventory_location import InventoryLocation
        from ..models.unit import Unit

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        _inventory_location = d.pop("inventory_location", UNSET)
        inventory_location: InventoryLocation | Unset
        if isinstance(_inventory_location, Unset):
            inventory_location = UNSET
        else:
            inventory_location = InventoryLocation.from_dict(_inventory_location)

        def _parse_sub_location(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        sub_location = _parse_sub_location(d.pop("sub_location", UNSET))

        def _parse_code(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        code = _parse_code(d.pop("code", UNSET))

        _food = d.pop("food", UNSET)
        food: Food | Unset
        if isinstance(_food, Unset):
            food = UNSET
        else:
            food = Food.from_dict(_food)

        _unit = d.pop("unit", UNSET)
        unit: Unit | Unset
        if isinstance(_unit, Unset):
            unit = UNSET
        else:
            unit = Unit.from_dict(_unit)

        amount = d.pop("amount", UNSET)

        def _parse_expires(data: object) -> datetime.date | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                expires_type_0 = isoparse(data).date()

                return expires_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.date | None | Unset, data)

        expires = _parse_expires(d.pop("expires", UNSET))

        def _parse_note(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        note = _parse_note(d.pop("note", UNSET))

        label = d.pop("label", UNSET)

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        created_by = d.pop("created_by", UNSET)

        patched_inventory_entry = cls(
            id=id,
            inventory_location=inventory_location,
            sub_location=sub_location,
            code=code,
            food=food,
            unit=unit,
            amount=amount,
            expires=expires,
            note=note,
            label=label,
            created_at=created_at,
            created_by=created_by,
        )

        patched_inventory_entry.additional_properties = d
        return patched_inventory_entry

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.recipe_from_source import RecipeFromSource
from ...models.recipe_from_source_response import RecipeFromSourceResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    body: RecipeFromSource | RecipeFromSource | RecipeFromSource | Unset = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/api/recipe-from-source/",
    }

    if isinstance(body, RecipeFromSource):
        if not isinstance(body, Unset):
            _kwargs["json"] = body.to_dict()

        headers["Content-Type"] = "application/json"
    if isinstance(body, RecipeFromSource):
        if not isinstance(body, Unset):
            _kwargs["data"] = body.to_dict()

        headers["Content-Type"] = "application/x-www-form-urlencoded"
    if isinstance(body, RecipeFromSource):
        if not isinstance(body, Unset):
            _kwargs["files"] = body.to_multipart()

        headers["Content-Type"] = "multipart/form-data"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> RecipeFromSourceResponse | None:
    if response.status_code == 200:
        response_200 = RecipeFromSourceResponse.from_dict(response.json())

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[RecipeFromSourceResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    body: RecipeFromSource | RecipeFromSource | RecipeFromSource | Unset = UNSET,
) -> Response[RecipeFromSourceResponse]:
    """function to retrieve a recipe from a given url or source string
    :param request: standard request with additional post parameters
            - url: url to use for importing recipe
            - data: if no url is given recipe is imported from provided source data
            - (optional) bookmarklet: id of bookmarklet import to use, overrides URL and data attributes
    :return: JsonResponse containing the parsed json and images

    Args:
        body (RecipeFromSource | Unset):
        body (RecipeFromSource | Unset):
        body (RecipeFromSource | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[RecipeFromSourceResponse]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    body: RecipeFromSource | RecipeFromSource | RecipeFromSource | Unset = UNSET,
) -> RecipeFromSourceResponse | None:
    """function to retrieve a recipe from a given url or source string
    :param request: standard request with additional post parameters
            - url: url to use for importing recipe
            - data: if no url is given recipe is imported from provided source data
            - (optional) bookmarklet: id of bookmarklet import to use, overrides URL and data attributes
    :return: JsonResponse containing the parsed json and images

    Args:
        body (RecipeFromSource | Unset):
        body (RecipeFromSource | Unset):
        body (RecipeFromSource | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        RecipeFromSourceResponse
    """

    return sync_detailed(
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    body: RecipeFromSource | RecipeFromSource | RecipeFromSource | Unset = UNSET,
) -> Response[RecipeFromSourceResponse]:
    """function to retrieve a recipe from a given url or source string
    :param request: standard request with additional post parameters
            - url: url to use for importing recipe
            - data: if no url is given recipe is imported from provided source data
            - (optional) bookmarklet: id of bookmarklet import to use, overrides URL and data attributes
    :return: JsonResponse containing the parsed json and images

    Args:
        body (RecipeFromSource | Unset):
        body (RecipeFromSource | Unset):
        body (RecipeFromSource | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[RecipeFromSourceResponse]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    body: RecipeFromSource | RecipeFromSource | RecipeFromSource | Unset = UNSET,
) -> RecipeFromSourceResponse | None:
    """function to retrieve a recipe from a given url or source string
    :param request: standard request with additional post parameters
            - url: url to use for importing recipe
            - data: if no url is given recipe is imported from provided source data
            - (optional) bookmarklet: id of bookmarklet import to use, overrides URL and data attributes
    :return: JsonResponse containing the parsed json and images

    Args:
        body (RecipeFromSource | Unset):
        body (RecipeFromSource | Unset):
        body (RecipeFromSource | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        RecipeFromSourceResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="SourceImportKeyword")


@_attrs_define
class SourceImportKeyword:
    """
    Attributes:
        label (str):
        name (str):
        id (int | None | Unset):
        import_keyword (bool | Unset):  Default: True.
    """

    label: str
    name: str
    id: int | None | Unset = UNSET
    import_keyword: bool | Unset = True
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        label = self.label

        name = self.name

        id: int | None | Unset
        if isinstance(self.id, Unset):
            id = UNSET
        else:
            id = self.id

        import_keyword = self.import_keyword

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "label": label,
                "name": name,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if import_keyword is not UNSET:
            field_dict["import_keyword"] = import_keyword

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        label = d.pop("label")

        name = d.pop("name")

        def _parse_id(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        id = _parse_id(d.pop("id", UNSET))

        import_keyword = d.pop("import_keyword", UNSET)

        source_import_keyword = cls(
            label=label,
            name=name,
            id=id,
            import_keyword=import_keyword,
        )

        source_import_keyword.additional_properties = d
        return source_import_keyword

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.storage import Storage


T = TypeVar("T", bound="PatchedRecipeImport")


@_attrs_define
class PatchedRecipeImport:
    """Adds nested create feature

    Attributes:
        id (int | Unset):
        storage (Storage | Unset): Adds nested create feature
        name (str | Unset):
        file_uid (str | Unset):
        file_path (str | Unset):
        created_at (datetime.datetime | Unset):
    """

    id: int | Unset = UNSET
    storage: Storage | Unset = UNSET
    name: str | Unset = UNSET
    file_uid: str | Unset = UNSET
    file_path: str | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        storage: dict[str, Any] | Unset = UNSET
        if not isinstance(self.storage, Unset):
            storage = self.storage.to_dict()

        name = self.name

        file_uid = self.file_uid

        file_path = self.file_path

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if storage is not UNSET:
            field_dict["storage"] = storage
        if name is not UNSET:
            field_dict["name"] = name
        if file_uid is not UNSET:
            field_dict["file_uid"] = file_uid
        if file_path is not UNSET:
            field_dict["file_path"] = file_path
        if created_at is not UNSET:
            field_dict["created_at"] = created_at

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.storage, Unset):
            files.append(("storage", (None, json.dumps(self.storage.to_dict()).encode(), "application/json")))

        if not isinstance(self.name, Unset):
            files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.file_uid, Unset):
            files.append(("file_uid", (None, str(self.file_uid).encode(), "text/plain")))

        if not isinstance(self.file_path, Unset):
            files.append(("file_path", (None, str(self.file_path).encode(), "text/plain")))

        if not isinstance(self.created_at, Unset):
            files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.storage import Storage

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        _storage = d.pop("storage", UNSET)
        storage: Storage | Unset
        if isinstance(_storage, Unset):
            storage = UNSET
        else:
            storage = Storage.from_dict(_storage)

        name = d.pop("name", UNSET)

        file_uid = d.pop("file_uid", UNSET)

        file_path = d.pop("file_path", UNSET)

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        patched_recipe_import = cls(
            id=id,
            storage=storage,
            name=name,
            file_uid=file_uid,
            file_path=file_path,
            created_at=created_at,
        )

        patched_recipe_import.additional_properties = d
        return patched_recipe_import

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="PatchedBookmarkletImport")


@_attrs_define
class PatchedBookmarkletImport:
    """
    Attributes:
        id (int | Unset):
        url (None | str | Unset):
        html (str | Unset):
        created_by (int | Unset):
        created_at (datetime.datetime | Unset):
    """

    id: int | Unset = UNSET
    url: None | str | Unset = UNSET
    html: str | Unset = UNSET
    created_by: int | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        url: None | str | Unset
        if isinstance(self.url, Unset):
            url = UNSET
        else:
            url = self.url

        html = self.html

        created_by = self.created_by

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if url is not UNSET:
            field_dict["url"] = url
        if html is not UNSET:
            field_dict["html"] = html
        if created_by is not UNSET:
            field_dict["created_by"] = created_by
        if created_at is not UNSET:
            field_dict["created_at"] = created_at

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.url, Unset):
            if isinstance(self.url, str):
                files.append(("url", (None, str(self.url).encode(), "text/plain")))
            else:
                files.append(("url", (None, str(self.url).encode(), "text/plain")))

        if not isinstance(self.html, Unset):
            files.append(("html", (None, str(self.html).encode(), "text/plain")))

        if not isinstance(self.created_by, Unset):
            files.append(("created_by", (None, str(self.created_by).encode(), "text/plain")))

        if not isinstance(self.created_at, Unset):
            files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        def _parse_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        url = _parse_url(d.pop("url", UNSET))

        html = d.pop("html", UNSET)

        created_by = d.pop("created_by", UNSET)

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        patched_bookmarklet_import = cls(
            id=id,
            url=url,
            html=html,
            created_by=created_by,
            created_at=created_at,
        )

        patched_bookmarklet_import.additional_properties = d
        return patched_bookmarklet_import

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.food import Food
    from ..models.inventory_location import InventoryLocation
    from ..models.unit import Unit


T = TypeVar("T", bound="InventoryEntry")


@_attrs_define
class InventoryEntry:
    """Adds nested create feature

    Attributes:
        inventory_location (InventoryLocation): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        food (Food): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        unit (Unit): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        label (str):
        created_at (datetime.datetime):
        created_by (int):
        id (int | Unset):
        sub_location (None | str | Unset):
        code (None | str | Unset):
        amount (float | Unset):
        expires (datetime.date | None | Unset):
        note (None | str | Unset):
    """

    inventory_location: InventoryLocation
    food: Food
    unit: Unit
    label: str
    created_at: datetime.datetime
    created_by: int
    id: int | Unset = UNSET
    sub_location: None | str | Unset = UNSET
    code: None | str | Unset = UNSET
    amount: float | Unset = UNSET
    expires: datetime.date | None | Unset = UNSET
    note: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        inventory_location = self.inventory_location.to_dict()

        food = self.food.to_dict()

        unit = self.unit.to_dict()

        label = self.label

        created_at = self.created_at.isoformat()

        created_by = self.created_by

        id = self.id

        sub_location: None | str | Unset
        if isinstance(self.sub_location, Unset):
            sub_location = UNSET
        else:
            sub_location = self.sub_location

        code: None | str | Unset
        if isinstance(self.code, Unset):
            code = UNSET
        else:
            code = self.code

        amount = self.amount

        expires: None | str | Unset
        if isinstance(self.expires, Unset):
            expires = UNSET
        elif isinstance(self.expires, datetime.date):
            expires = self.expires.isoformat()
        else:
            expires = self.expires

        note: None | str | Unset
        if isinstance(self.note, Unset):
            note = UNSET
        else:
            note = self.note

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "inventory_location": inventory_location,
                "food": food,
                "unit": unit,
                "label": label,
                "created_at": created_at,
                "created_by": created_by,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if sub_location is not UNSET:
            field_dict["sub_location"] = sub_location
        if code is not UNSET:
            field_dict["code"] = code
        if amount is not UNSET:
            field_dict["amount"] = amount
        if expires is not UNSET:
            field_dict["expires"] = expires
        if note is not UNSET:
            field_dict["note"] = note

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(
            ("inventory_location", (None, json.dumps(self.inventory_location.to_dict()).encode(), "application/json"))
        )

        files.append(("food", (None, json.dumps(self.food.to_dict()).encode(), "application/json")))

        files.append(("unit", (None, json.dumps(self.unit.to_dict()).encode(), "application/json")))

        files.append(("label", (None, str(self.label).encode(), "text/plain")))

        files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        files.append(("created_by", (None, str(self.created_by).encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.sub_location, Unset):
            if isinstance(self.sub_location, str):
                files.append(("sub_location", (None, str(self.sub_location).encode(), "text/plain")))
            else:
                files.append(("sub_location", (None, str(self.sub_location).encode(), "text/plain")))

        if not isinstance(self.code, Unset):
            if isinstance(self.code, str):
                files.append(("code", (None, str(self.code).encode(), "text/plain")))
            else:
                files.append(("code", (None, str(self.code).encode(), "text/plain")))

        if not isinstance(self.amount, Unset):
            files.append(("amount", (None, str(self.amount).encode(), "text/plain")))

        if not isinstance(self.expires, Unset):
            if isinstance(self.expires, datetime.date):
                files.append(("expires", (None, self.expires.isoformat().encode(), "text/plain")))
            else:
                files.append(("expires", (None, str(self.expires).encode(), "text/plain")))

        if not isinstance(self.note, Unset):
            if isinstance(self.note, str):
                files.append(("note", (None, str(self.note).encode(), "text/plain")))
            else:
                files.append(("note", (None, str(self.note).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.food import Food
        from ..models.inventory_location import InventoryLocation
        from ..models.unit import Unit

        d = dict(src_dict)
        inventory_location = InventoryLocation.from_dict(d.pop("inventory_location"))

        food = Food.from_dict(d.pop("food"))

        unit = Unit.from_dict(d.pop("unit"))

        label = d.pop("label")

        created_at = isoparse(d.pop("created_at"))

        created_by = d.pop("created_by")

        id = d.pop("id", UNSET)

        def _parse_sub_location(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        sub_location = _parse_sub_location(d.pop("sub_location", UNSET))

        def _parse_code(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        code = _parse_code(d.pop("code", UNSET))

        amount = d.pop("amount", UNSET)

        def _parse_expires(data: object) -> datetime.date | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                expires_type_0 = isoparse(data).date()

                return expires_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.date | None | Unset, data)

        expires = _parse_expires(d.pop("expires", UNSET))

        def _parse_note(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        note = _parse_note(d.pop("note", UNSET))

        inventory_entry = cls(
            inventory_location=inventory_location,
            food=food,
            unit=unit,
            label=label,
            created_at=created_at,
            created_by=created_by,
            id=id,
            sub_location=sub_location,
            code=code,
            amount=amount,
            expires=expires,
            note=note,
        )

        inventory_entry.additional_properties = d
        return inventory_entry

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.paginated_shopping_list_entry_list import PaginatedShoppingListEntryList
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    mealplan: int | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    updated_after: datetime.datetime | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    params["mealplan"] = mealplan

    params["page"] = page

    params["page_size"] = page_size

    json_updated_after: str | Unset = UNSET
    if not isinstance(updated_after, Unset):
        json_updated_after = updated_after.isoformat()
    params["updated_after"] = json_updated_after

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/shopping-list-entry/",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> PaginatedShoppingListEntryList | None:
    if response.status_code == 200:
        response_200 = PaginatedShoppingListEntryList.from_dict(response.json())

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[PaginatedShoppingListEntryList]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    mealplan: int | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    updated_after: datetime.datetime | Unset = UNSET,
) -> Response[PaginatedShoppingListEntryList]:
    """individual entries of a shopping list
    automatically filtered to only contain unchecked items that are not older than the shopping recent
    days setting to not bloat endpoint

    Args:
        mealplan (int | Unset):
        page (int | Unset):
        page_size (int | Unset):
        updated_after (datetime.datetime | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedShoppingListEntryList]
    """

    kwargs = _get_kwargs(
        mealplan=mealplan,
        page=page,
        page_size=page_size,
        updated_after=updated_after,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    mealplan: int | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    updated_after: datetime.datetime | Unset = UNSET,
) -> PaginatedShoppingListEntryList | None:
    """individual entries of a shopping list
    automatically filtered to only contain unchecked items that are not older than the shopping recent
    days setting to not bloat endpoint

    Args:
        mealplan (int | Unset):
        page (int | Unset):
        page_size (int | Unset):
        updated_after (datetime.datetime | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedShoppingListEntryList
    """

    return sync_detailed(
        client=client,
        mealplan=mealplan,
        page=page,
        page_size=page_size,
        updated_after=updated_after,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    mealplan: int | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    updated_after: datetime.datetime | Unset = UNSET,
) -> Response[PaginatedShoppingListEntryList]:
    """individual entries of a shopping list
    automatically filtered to only contain unchecked items that are not older than the shopping recent
    days setting to not bloat endpoint

    Args:
        mealplan (int | Unset):
        page (int | Unset):
        page_size (int | Unset):
        updated_after (datetime.datetime | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedShoppingListEntryList]
    """

    kwargs = _get_kwargs(
        mealplan=mealplan,
        page=page,
        page_size=page_size,
        updated_after=updated_after,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    mealplan: int | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    updated_after: datetime.datetime | Unset = UNSET,
) -> PaginatedShoppingListEntryList | None:
    """individual entries of a shopping list
    automatically filtered to only contain unchecked items that are not older than the shopping recent
    days setting to not bloat endpoint

    Args:
        mealplan (int | Unset):
        page (int | Unset):
        page_size (int | Unset):
        updated_after (datetime.datetime | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedShoppingListEntryList
    """

    return (
        await asyncio_detailed(
            client=client,
            mealplan=mealplan,
            page=page,
            page_size=page_size,
            updated_after=updated_after,
        )
    ).parsed



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="PatchedHousehold")


@_attrs_define
class PatchedHousehold:
    """Adds nested create feature

    Attributes:
        id (int | Unset):
        name (str | Unset):
        created_at (datetime.datetime | Unset):
        updated_at (datetime.datetime | Unset):
    """

    id: int | Unset = UNSET
    name: str | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    updated_at: datetime.datetime | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: str | Unset = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.name, Unset):
            files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.created_at, Unset):
            files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.updated_at, Unset):
            files.append(("updated_at", (None, self.updated_at.isoformat().encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: datetime.datetime | Unset
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        patched_household = cls(
            id=id,
            name=name,
            created_at=created_at,
            updated_at=updated_at,
        )

        patched_household.additional_properties = d
        return patched_household

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
